import { ValidationContext } from '@common/forms/validation/validation-context';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { BackendErrorValidator } from '@common/forms/validation/custom-validator';

export class ExtraPropertiesFormModel {
	public language: string;
	public license: string;
	public visible: boolean;
	public publicDate: Date;

	fromModel(item: any): ExtraPropertiesFormModel {
		this.language = item.language;
		this.license = item.license;
		this.visible = item.visible;
		this.publicDate = item.publicDate;
		return this;
	}

	buildForm(context: ValidationContext = null, disabled: boolean = false): FormGroup {
		if (context == null) { context = this.createValidationContext(); }

		const formGroup = new FormBuilder().group({
			language: [{ value: this.language, disabled: disabled }, context.getValidation('language').validators],
			license: [{ value: this.license, disabled: disabled }, context.getValidation('license').validators],
			visible: [{ value: this.visible, disabled: disabled }, context.getValidation('visible').validators],
			publicDate: [{ value: this.publicDate, disabled: disabled }, context.getValidation('publicDate').validators]
		});
		return formGroup;
	}

	createValidationContext(): ValidationContext {
		const baseContext: ValidationContext = new ValidationContext();
		baseContext.validation.push({ key: 'language', validators: [] });
		baseContext.validation.push({ key: 'license', validators: [] });
		baseContext.validation.push({ key: 'visible', validators: [] });
		baseContext.validation.push({ key: 'publicDate', validators: [] });
		return baseContext;
	}

}
